--[[
	Title: Utilities

	Has some useful server utilities
]]

--[[
	Function: sendLuaFunction

	Makes sending function calls a breeze. Will automatically quotes strings, tostring bools, etc.

	Parameters:

		ply - The player to send to, set to nil to send to everyone.
		fn - The function to call (Must be a string!)
		... - Any extra number of arguments.
]]
function ULib.sendLuaFunction( ply, fn, ... )
	local str = ""
	local args = { ... }
	for _, v in pairs( args ) do
		str = str .. ULib.serialize( v ) .. ","
	end
	str = str:sub( 1, -2 ) -- Clip off last ","

	-- Now for the finishing touches.
	str = fn .. "(" .. str .. ")"

	if ply then
		ply:SendLua( str )
	else
		local players = player.GetAll()
		for _, player in ipairs( players ) do
			player:SendLua( str )
		end
	end
end

--[[
	Function: umsgSend

	Makes sending umsgs a blast. You don't have to bother knowing what type you're sending, just use ULib.umsgRcv() on the client.

	Parameters:

		v - The type to send
]]
function ULib.umsgSend( v )
	local tv = type( v )

	if tv == "string" then
		umsg.Short( ULib.TYPE_STRING )
		umsg.String( v )
	elseif tv == "number" then
		if math.fmod( v, 1 ) ~= 0 then -- It's a float
			umsg.Short( ULib.TYPE_FLOAT )
			umsg.Float( v )
		else
			if v < 32767 and v > -32768 then
				umsg.Short( ULib.TYPE_SHORT )
				umsg.Short( v )
			else
				umsg.Short( ULib.TYPE_LONG )
				umsg.Long( v )
			end
		end
	elseif tv == "boolean" then
		umsg.Short( ULib.TYPE_BOOLEAN )
		umsg.Bool( v )
	elseif tv == "Entity" then
		umsg.Short( ULib.TYPE_ENTITY )
		umsg.Entity( v )
	elseif tv == "Player" then
		umsg.Short( ULib.TYPE_PLAYER )
		umsg.Short( v:EntIndex() )
	elseif tv == "Vector" then
		umsg.Short( ULib.TYPE_VECTOR )
		umsg.Vector( v )
	elseif tv == "Angle" then
		umsg.Short( ULib.TYPE_ANGLE )
		umsg.Vector( Vector( v.pitch, v.yaw, v.roll ) )
	elseif tv == "table" then
		umsg.Short( ULib.TYPE_TABLE_BEGIN )
		for key, value in pairs( v ) do
			ULib.umsgSend( key )
			ULib.umsgSend( value )
		end
		umsg.Short( ULib.TYPE_TABLE_END )
	elseif tv == "nil" then
		umsg.Short( ULib.TYPE_NIL )
	else
		ULib.error( "Unknown type passed to umsgSend -- " .. tv )
	end
end


--[[
	Function: play3DSound

	Plays a 3D sound, the further away from the point the player is, the softer the sound will be.

	Parameters:

		sound - The sound to play, relative to the sound folder.
		vector - The point to play the sound at.
		volume - *(Optional, defaults to 1)* The volume to make the sound.
		pitch - *(Optional, defaults to 1)* The pitch to make the sound, 1 = normal.
]]
function ULib.play3DSound( sound, vector, volume, pitch )
	volume = volume or 100
	pitch = pitch or 100

	local ent = ents.Create( "info_null" )
	if not ent:IsValid() then return end
	ent:SetPos( vector )
	ent:Spawn()
	ent:Activate()
	ent:EmitSound( sound, volume, pitch )
end


--[[
	Function: isDedicatedServer

	The normal GM10 bind is broken, this fixes it.
]]
function isDedicatedServer() -- The normal GM10 function isn't working, let's define a hack
	return GetConVarString( "sensitivity" ) == "" -- This doesn't exist on a ded server, but exists on a listen server
end

local meta = FindMetaTable( "Player" )

if file.Exists( "../lua/Admin/Core.lua" ) then -- Because I have nothing better to do than clean up after Conna's inept coding...
	meta.IsListenServerHost = meta.ListenSH -- Imagine that, he stole a ded server IsListenServerHost() patch WITHOUT EVEN TESTING IT!
end


-- Here's a quick patch so dedicated servers aren't spammed with messages about using IsListenServerHost()
-- Unlike Conna's stolen patch, this one actually works, and takes less CPU time.
if isDedicatedServer() then
	// Return if there's nothing to add on to
	if not meta then return end


	function meta:IsListenServerHost()
	        return false
	end
end