//========= Copyright  1996-2005, Valve Corporation, All rights reserved. ============//
//
// Purpose: 
//
// $NoKeywords: $
//
//=============================================================================//

#include "ulib_plugin.h"
#include "common/GMLuaModule.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

// Interfaces from the engine
IVEngineServer	*engine = NULL; // helper functions (messaging clients, loading content, making entities, running commands, etc)
IFileSystem		*filesystem = NULL; // file I/O 
IGameEventManager *gameeventmanager = NULL; // game events interface
IPlayerInfoManager *playerinfomanager = NULL; // game dll interface to interact with players
IBotManager *botmanager = NULL; // game dll interface to interact with bots
IServerPluginHelpers *helpers = NULL; // special 3rd party plugin helpers from the engine
IUniformRandomStream *randomStr = NULL;
IEngineTrace *enginetrace = NULL;
INetworkStringTableContainer *networkStringTable = NULL; // We use this for TYPE_INDEX motd's


CGlobalVars *gpGlobals = NULL;

// function to initialize any cvars/command in this plugin
void InitCVars( CreateInterfaceFn cvarFactory );
enum motd_type { TYPE_TEXT=1, TYPE_INDEX, TYPE_URL, TYPE_FILE };
void ShowMOTD(int index, const char *title, const char *msg, motd_type type, const char *cmd=NULL);


// 
// The plugin is a static singleton that is exported as an interface
//
CEmptyServerPlugin g_EmtpyServerPlugin;
EXPOSE_SINGLE_INTERFACE_GLOBALVAR(CEmptyServerPlugin, IServerPluginCallbacks, INTERFACEVERSION_ISERVERPLUGINCALLBACKS, g_EmtpyServerPlugin );


//---------------------------------------------------------------------------------
// Purpose: constructor/destructor
//---------------------------------------------------------------------------------
CEmptyServerPlugin::CEmptyServerPlugin()
{
	m_iClientCommandIndex = 0;
}

CEmptyServerPlugin::~CEmptyServerPlugin()
{
}

//---------------------------------------------------------------------------------
// Purpose: called when the plugin is loaded, load the interface we need from the engine
//---------------------------------------------------------------------------------
bool CEmptyServerPlugin::Load(	CreateInterfaceFn interfaceFactory, CreateInterfaceFn gameServerFactory )
{
	playerinfomanager = (IPlayerInfoManager *)gameServerFactory(INTERFACEVERSION_PLAYERINFOMANAGER,NULL);
	if ( !playerinfomanager )
	{
		Warning( "Unable to load playerinfomanager, ignoring\n" ); // this isn't fatal, we just won't be able to access specific player data
	}

	// get the interfaces we want to use
	if(	!(cvar = (ICvar*)interfaceFactory(VENGINE_CVAR_INTERFACE_VERSION, NULL)) ||
		!(engine = (IVEngineServer*)interfaceFactory(INTERFACEVERSION_VENGINESERVER, NULL)) ||
		!(gameeventmanager = (IGameEventManager *)interfaceFactory(INTERFACEVERSION_GAMEEVENTSMANAGER,NULL)) ||
		!(filesystem = (IFileSystem*)interfaceFactory(FILESYSTEM_INTERFACE_VERSION, NULL)) ||
		!(helpers = (IServerPluginHelpers*)interfaceFactory(INTERFACEVERSION_ISERVERPLUGINHELPERS, NULL)) || 
		!(enginetrace = (IEngineTrace *)interfaceFactory(INTERFACEVERSION_ENGINETRACE_SERVER,NULL)) ||
		!(randomStr = (IUniformRandomStream *)interfaceFactory(VENGINE_SERVER_RANDOM_INTERFACE_VERSION, NULL)) ||
		!(networkStringTable = (INetworkStringTableContainer *)interfaceFactory(INTERFACENAME_NETWORKSTRINGTABLESERVER, NULL))
		)
	{
		return false; // we require all these interface to function
	}

	if ( playerinfomanager )
	{
		gpGlobals = playerinfomanager->GetGlobalVars();
	}

	InitCVars( interfaceFactory ); // register any cvars we have defined


	/*IServerGameDLL *serverdll = NULL;
if ( !(serverdll = (IServerGameDLL *)gameServerFactory( INTERFACEVERSION_SERVERGAMEDLL, NULL ) ) ) {
    Msg( "Failed to initialize serverdll\n" );
} else {
    for( int i = 0; i <	17; i++ ) {
        char msgName[32];
        int sizereturn = 0;
        if ( serverdll->GetUserMessageInfo( i, msgName, 31, sizereturn ) ) {
            Msg( "User Message \"%s\" is index %i.\n", msgName, i );
            // Do something to keep track of the indexes, like store it in a list
        }
    }
}*/


	return true;
}

//---------------------------------------------------------------------------------
// Purpose: called when the plugin is unloaded (turned off)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::Unload( void )
{
	gameeventmanager->RemoveListener( this ); // make sure we are unloaded from the event system
}

//---------------------------------------------------------------------------------
// Purpose: called when the plugin is paused (i.e should stop running but isn't unloaded)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::Pause( void )
{
}

//---------------------------------------------------------------------------------
// Purpose: called when the plugin is unpaused (i.e should start executing again)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::UnPause( void )
{
}

//---------------------------------------------------------------------------------
// Purpose: the name of this plugin, returned in "plugin_print" command
//---------------------------------------------------------------------------------
const char *CEmptyServerPlugin::GetPluginDescription( void )
{
	return "MOTD plugin.";
}

//---------------------------------------------------------------------------------
// Purpose: called on level start
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::LevelInit( char const *pMapName )
{
	gameeventmanager->AddListener( this, true );
}

//---------------------------------------------------------------------------------
// Purpose: called on level start, when the server is ready to accept client connections
//		edictCount is the number of entities in the level, clientMax is the max client count
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::ServerActivate( edict_t *pEdictList, int edictCount, int clientMax )
{
}

//---------------------------------------------------------------------------------
// Purpose: called once per server frame, do recurring work here (like checking for timeouts)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::GameFrame( bool simulating )
{
}

//---------------------------------------------------------------------------------
// Purpose: called on level end (as the server is shutting down or going to a new map)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::LevelShutdown( void ) // !!!!this can get called multiple times per map change
{
	gameeventmanager->RemoveListener( this );
}

//---------------------------------------------------------------------------------
// Purpose: called when a client spawns into a server (i.e as they begin to play)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::ClientActive( edict_t *pEntity )
{
}

//---------------------------------------------------------------------------------
// Purpose: called when a client leaves a server (or is timed out)
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::ClientDisconnect( edict_t *pEntity )
{
}

//---------------------------------------------------------------------------------
// Purpose: called on 
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::ClientPutInServer( edict_t *pEntity, char const *playername )
{
}

//---------------------------------------------------------------------------------
// Purpose: called on
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::SetCommandClient( int index )
{
	m_iClientCommandIndex = index;
}

//---------------------------------------------------------------------------------
// Purpose: called on level start
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::ClientSettingsChanged( edict_t *pEdict )
{
}

//---------------------------------------------------------------------------------
// Purpose: called when a client joins a server
//---------------------------------------------------------------------------------
PLUGIN_RESULT CEmptyServerPlugin::ClientConnect( bool *bAllowConnect, edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen )
{
	return PLUGIN_CONTINUE;
}

//---------------------------------------------------------------------------------
// Purpose: called when a client types in a command (only a subset of commands however, not CON_COMMAND's)
//---------------------------------------------------------------------------------
PLUGIN_RESULT CEmptyServerPlugin::ClientCommand( edict_t *pEntity )
{
	//const char *pcmd = engine->Cmd_Argv(0);
	//Msg( "Command from user: %s\n", pcmd );
	return PLUGIN_CONTINUE;
}

//---------------------------------------------------------------------------------
// Purpose: called when a client is authenticated
//---------------------------------------------------------------------------------
PLUGIN_RESULT CEmptyServerPlugin::NetworkIDValidated( const char *pszUserName, const char *pszNetworkID )
{
	return PLUGIN_CONTINUE;
}

//---------------------------------------------------------------------------------
// Purpose: called when an event is fired
//---------------------------------------------------------------------------------
void CEmptyServerPlugin::FireGameEvent( KeyValues * event )
{
	//const char * name = event->GetName();
	//Msg( "CEmptyServerPlugin::FireGameEvent: Got event \"%s\"\n", name );
}

void ShowMOTD(int index, const char *title, const char *msg, motd_type type, const char *cmd) {
	bf_write *buffer;
	MRecipientFilter filter;

	filter.AddRecipient(index);

	buffer = engine->UserMessageBegin(&filter, 12);
	buffer->WriteString("info");
	buffer->WriteByte(1);

	if(cmd != NULL)
		buffer->WriteByte(4);
	else
		buffer->WriteByte(3);

	buffer->WriteString("title");
	buffer->WriteString(title);

	buffer->WriteString("type");
	switch(type) {
		case TYPE_TEXT:
			buffer->WriteString("0"); //TYPE_TEXT = 0, just display this plain text
			break;

		case TYPE_INDEX:
			buffer->WriteString("1"); //TYPE_INDEX, lookup text & title in stringtable
			break;

		case TYPE_URL:
			buffer->WriteString("2"); //TYPE_URL, show this URL
			break;

		case TYPE_FILE:
			buffer->WriteString("3"); //TYPE_FILE, show this local file
			break;
	}

	buffer->WriteString("msg");
	buffer->WriteString(msg); // msg must not be greater than 192 characters

	if(cmd != NULL) {
		buffer->WriteString("cmd");
		buffer->WriteString(cmd); // exec this command if panel closed
	}

	engine->MessageEnd();
	return ;
}

// This is a macro to make life easier.
// When the dll is loaded it will call the function (Init)
GMOD_MODULE( Init );

#define CHECK_ARG( POS, TYPE, STR ) if ( g_Lua->GetType( POS ) != TYPE ) { \
	char buf[ 64 ];\
	sprintf( buf, "%s expected, got %s", STR, g_Lua->GetType( POS ) == GLua::TYPE_NIL ? "no value" : g_Lua->GetTypeName( POS ) );\
	g_Lua->LuaError( buf, POS );\
}

LUA_FUNCTION( ShowMOTD )
{
	CHECK_ARG( 1, GLua::TYPE_NUMBER, "number" );
	CHECK_ARG( 2, GLua::TYPE_STRING, "string" );
	CHECK_ARG( 3, GLua::TYPE_STRING, "string" );
	CHECK_ARG( 4, GLua::TYPE_NUMBER, "number" );
	if ( g_Lua->GetType( 5 ) != GLua::TYPE_NIL && g_Lua->GetType( 5 ) != GLua::TYPE_STRING )
	{
		CHECK_ARG( 5, GLua::TYPE_NIL, "nil or string" );
	}

	const int index = g_Lua->GetInteger( 1 );
	const char *title = g_Lua->GetString( 2 );
	const char *msg = g_Lua->GetString( 3 );
	motd_type type = (motd_type)g_Lua->GetInteger( 4 );
	const char *cmd = g_Lua->GetString( 5 );

	if ( (int)type > 4 || (int)type < 1 )
	{
		g_Lua->LuaError( "type out of range", 4 );
	}

	if ( type == TYPE_INDEX )
	{

		INetworkStringTable *pInfoPanel = networkStringTable->FindTable( "InfoPanel" );
		if ( pInfoPanel )
		{
			bool save = engine->LockNetworkStringTables( false );
			int StrIdx = pInfoPanel->AddString( "myinfo" );
			pInfoPanel->SetStringUserData( StrIdx, strlen( msg ), msg );
			engine->LockNetworkStringTables( save );
		}

		ShowMOTD( index, title, "myinfo", TYPE_INDEX, cmd );
	}
	else
	{
		ShowMOTD( index, title, msg, type, cmd );
	}

	return 0;
}

// When the DLL is initialized it will call this function to set itself up.
// It should be used to set up your library so it's all ready to go.
int Init( void )
{
	// Register our MsgBox function as a global
	g_Lua->SetGlobal( "ShowMOTD", ShowMOTD );
	g_Lua->SetGlobal( "TYPE_TEXT", (float)1 );
	g_Lua->SetGlobal( "TYPE_INDEX", (float)2 );
	g_Lua->SetGlobal( "TYPE_URL", (float)3 );
	g_Lua->SetGlobal( "TYPE_FILE", (float)4 );

	// Lets make a global table filled with functions (a module)
	/*g_Lua->NewGlobalTable( "my_module" );
	ILuaObject* table = g_Lua->GetGlobal( "my_module" );

		table->SetMember( "GetVariable", GetVariable );

		// You can register the same function twice with different names
		table->SetMember( "MsgBox", MsgBox );
		table->SetMember( "GetString", GetVariable );
		table->SetMember( "Test", Test );

	// We're not deleting the table here - we're deleting the reference to it.
	// If you don't do this you're leaking memory.
	SAFE_UNREF( table );*/

	return 0;
}
